#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "KOSMA_error.h"

#ifndef TRUE
  #define TRUE 1
#endif
#ifndef FALSE
  #define FALSE 0
#endif

#define INFOBLOCK  VAR_COMMENT comment; VAR_SET *read_var_set; int (*read_fct)(); VAR_SET_LIST write_var_set_list; int (*write_fct)(); char *write_format;

typedef char* VAR_NAME;
typedef char* VAR_COMMENT;


/* definitions of var_set related types */

typedef struct var_set_entry
{
  VAR_NAME *variable;
  struct var_set_entry *next;
} VAR_SET_ENTRY;

typedef struct
{
  char *filename;
  VAR_SET_ENTRY *first;
  VAR_SET_ENTRY *last;
} VAR_SET;

typedef struct var_set_list_entry
{
  VAR_SET *var_set;
  struct var_set_list_entry *next;
} VAR_SET_LIST_ENTRY;

typedef struct
{
  VAR_SET_LIST_ENTRY *first;
  VAR_SET_LIST_ENTRY *last;
} VAR_SET_LIST;


/* definitions of variable types */
        
typedef struct
{
  VAR_NAME name;
  INFOBLOCK
  int value;
} INT_VAR;
        
typedef struct
{
  VAR_NAME name;
  INFOBLOCK
  double value;
} DOUBLE_VAR;
        
typedef struct
{
  VAR_NAME name;
  INFOBLOCK
  char *value;
} STRING_VAR;


/* function prototypes */

/* configure an int variable */
int ConfigureInt( INT_VAR *variable,
                  char *name_string,
                  char *config_file_name,
                  char *dimension_string );
/* configure a double variable */
int ConfigureDouble( DOUBLE_VAR *variable,
                     char *name_string,
                     char *config_file_name,
                     char *dimension_string );
/* configure a string variable */
int ConfigureString( STRING_VAR *variable,
                     char *name_string,
                     char *config_file_name,
                     char *dimension_string );
/* produce a dimension_string as needed in Configure... calls from a comma
   separated list of integer. 0 terminates list. */
char *Dimension( int dim1, ... );

/* read all previously configured variables */
double ReadAllVariables();
/* read all variables from the file containing the variable VAR_NAME * */
double ReadVariablesVarSet( VAR_NAME *variable_name );
/* read one variable (ReadVariablesVarSet is the preferred function!) */
double ReadVar( VAR_NAME *variable_name );

/* write all previously configured variables  */
int WriteAllVariables();
/* write all variables in all output files containing the variable VAR_NAME * */
int WriteVariablesVarSet( VAR_NAME *variable_name );
/* same as above, USAGE NOT RECOMMENDED */
int WriteVar( VAR_NAME *variable_name ); 

/* set the default config file name */
void DefineConfigFile( char *config_file_name );
/* set the standard config file name (i.e. program file basename with extension    ".cfg") */
void StandardConfigFile( char *source_file_name_or_file_name_macro );
/* set directory paths (overrides environment variables) */
void SetConfigDir( char *config_dir_name );
void SetReadDir( char *read_dir_name );
void SetWriteDir( char *write_dir_name );
